#
# PAC_ENV_PETSC()
#
# Check for the existence of PETSc defines and set them to defaults
AC_DEFUN(PAC_ENV_PETSC,
[
if test -n "$PETSC_DIR"; then
    if test -z "$BOPT"; then
        BOPT="g"
    fi
    if test -z "$PETSC_ARCH"; then
        PETSC_ARCH=`$PETSC_DIR/bin/petscarch`
    fi
fi
]
)
# end of PAC_ENV_PETSC
# PAC_LIB_MPI_AUX([ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
#
# Check for auxilliary information about MPI
AC_DEFUN(PAC_LIB_MPI_AUX,
[
if test "$USE_MPI" = "yes"; then
    # Get sizeof MPI_Comm
    save_CPPFLAGS="$CPPFLAGS"
    CPPFLAGS="$CPPFLAGS -I$MPI_INCLUDE_DIR"
    PAC_CHECK_SIZEOF(MPI_Comm, , "mpi.h")
    CPPFLAGS="$save_CPPFLAGS"

    # Figure out conversions to Fortran
    save_CPPFLAGS="$CPPFLAGS"
    CPPFLAGS="$CPPFLAGS -I$MPI_INCLUDE_DIR"
    save_LIBS="$LIBS"
    LIBS="$LIBS $MPI_LIB"
    AC_MSG_CHECKING([MPI_Comm_f2c in -l$MPI_LIB_NAME])
    AC_TRY_LINK([#include "mpi.h"],
                [MPI_Comm_f2c(MPI_COMM_WORLD);],
                AC_MSG_RESULT([yes])
                AC_DEFINE(HAVE_MPI_COMM_F2C)
                ac_cv_lib_mpi_MPI_Comm_f2c="yes",
                AC_MSG_RESULT([no])
                ac_cv_lib_mpi_MPI_Comm_f2c="no")
    AC_MSG_CHECKING([MPI_Comm_c2f in -l$MPI_LIB_NAME])
    AC_TRY_LINK([#include "mpi.h"],
                [MPI_Comm_c2f(MPI_COMM_WORLD);],
                AC_MSG_RESULT([yes])
                AC_DEFINE(HAVE_MPI_COMM_C2F)
                ac_cv_lib_mpi_MPI_Comm_c2f="yes",
                AC_MSG_RESULT([no])
                ac_cv_lib_mpi_MPI_Comm_c2f="no")
    LIBS="$save_LIBS"
    CPPFLAGS="$save_CPPFLAGS"
fi
]
)
# end of PAC_LIB_MPI_AUX
#
# PAC_LIB_MPI([ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
#
# Check for the existence of MPI
AC_DEFUN(PAC_LIB_MPI,
[
AC_ARG_WITH(mpi, [  --with-mpi[=LIB]            Choose MPI implementation], [MPI_LIB=$withval])
AC_ARG_WITH(mpi-include, [  --with-mpi-include=DIR      Choose MPI include directory], [MPI_INCLUDE_DIR=$withval])
USE_MPI="no"
if test -n "$MPI_LIB"; then
    if test "$MPI_LIB" = "yes"; then
        USE_MPI="yes"
        MPI_LIB="mpich"
    elif test "$MPI_LIB" = "no"; then
        USE_MPI="no"
        MPI_LIB="mpich"
    else
        USE_MPI="yes"
    fi
fi
if test "$USE_MPI" = "yes"; then
    ac_cv_lib_mpi=no
    save_LIBS="$LIBS"

    if test -n "$MPI_LIB" && test ! "$MPI_LIB" = "yes"; then
        MPI_LIB_DIR=`dirname $MPI_LIB`
        if test ! "$MPI_LIB_DIR" = "."; then
            if test ! -f "$MPI_LIB" ; then
                AC_MSG_ERROR([MPI library $MPI_LIB does not exist!])
            fi
            # We need to strip off the /lib level
            MPI_DIR=`dirname $MPI_LIB_DIR`
        else
            # The user only gave the library name, not location
            MPI_DIR="/usr"
        fi
        temp_LIB=`basename $MPI_LIB .a`
        MPI_LIB=`basename $temp_LIB .so | sed -e s/^lib//`
        MPIRUN="$MPI_DIR/bin/mpirun"
        if test -z "$MPI_INCLUDE_DIR"; then
            MPI_INCLUDE_DIR="$MPI_DIR/include"
        fi
    else
        MPI_LIB=mpich
    fi

    # We can't use SEARCH_LIBS in this macro since it caches the value
    AC_CHECK_LIB($MPI_LIB, MPI_Init, , , [-L$MPI_LIB_DIR])
    MPI_LIB_CANONICAL=`echo $MPI_LIB | sed 'y%./+-%__p_%'`
    if eval "test \"`echo '$''{ac_cv_lib_'$MPI_LIB_CANONICAL'_MPI_Init}'`\" = no"; then
        MPI_LIB=mpi
        AC_CHECK_LIB(mpi, MPI_Init, , , [-L$MPI_LIB_DIR])
        # If we did not find the provided library name MPI_LIB, then hereafter we assume mpich/mpi
    else
        ac_cv_lib_mpi_MPI_Init=yes
    fi

    # Try using MPILIBPATH and MPIINCLUDEPATH
    if test "$ac_cv_lib_mpi_MPI_Init" = "no"; then
        unset ac_cv_lib_mpich_MPI_Init
        MPI_LIB=mpich
        AC_CHECK_LIB(mpich, MPI_Init, , , [-L$MPILIBPATH])
        if test "$ac_cv_lib_mpich_MPI_Init" = "no"; then
            unset ac_cv_lib_mpi_MPI_Init
            MPI_LIB=mpi
            AC_CHECK_LIB(mpi, MPI_Init, , , [-L$MPILIBPATH])
        else
            ac_cv_lib_mpi_MPI_Init=yes
        fi

        if test "$ac_cv_lib_mpi_MPI_Init" = "yes" ; then
            MPI_INCLUDE_DIR="$MPIINCLUDEPATH"
            MPI_LIB_DIR="$MPILIBPATH"
        fi
    fi

    # Try using MPI_DIR
    if test "$ac_cv_lib_mpi_MPI_Init" = "no" ; then
        unset ac_cv_lib_mpich_MPI_Init
        MPI_LIB=mpich
        AC_CHECK_LIB(mpich, MPI_Init, , , [-L$MPI_DIR/lib])
        if test "$ac_cv_lib_mpich_MPI_Init" = "no"; then
            unset ac_cv_lib_mpi_MPI_Init
            MPI_LIB=mpi
            AC_CHECK_LIB(mpi, MPI_Init, , , [-L$MPI_DIR/lib])
        else
            ac_cv_lib_mpi_MPI_Init=yes
        fi

        if test "$ac_cv_lib_mpi_MPI_Init" = "yes" ; then
            MPI_INCLUDE_DIR="$MPI_DIR/include"
            MPI_LIB_DIR="$MPI_DIR/lib"
        fi
    fi

    if test "$ac_cv_lib_mpi_MPI_Init" = "yes"; then
        # Here we assume that a user who provided the library will want an error if the header is missing
        save_CPPFLAGS="$CPPFLAGS"
        CPPFLAGS="$CPPFLAGS -I$MPI_INCLUDE_DIR"
        AC_CHECK_HEADER(mpi.h, , AC_MSG_ERROR([Could not find $MPI_INCLUDE_DIR/mpi.h!]))
        CPPFLAGS="$save_CPPFLAGS"
        ac_cv_lib_mpi=yes
    fi

    if test "$ac_cv_lib_mpi" = "yes"; then
        MPI_LIB_NAME="$MPI_LIB"
        LIBS="$save_LIBS"
        if test -z "$MPIRUN"; then
            MPIRUN="$MPI_DIR/bin/mpirun"
        fi
        if test -n "$MPI_LIB_DIR"; then
            LDFLAGS="$LDFLAGS -L$MPI_LIB_DIR"
            DYLIBPATH="$DYLIBPATH:$MPI_LIB_DIR"
        fi
        MPI_INCLUDE="-I$MPI_INCLUDE_DIR"
        MPI_LIB="-l$MPI_LIB"
        EXTRA_LIBS="$MPI_LIB $EXTRA_LIBS"

        PAC_LIB_MPI_AUX

        ifelse([$1], , [
               AC_DEFINE(HAVE_MPI)
               AC_SUBST(MPI_LIB)
               AC_SUBST(MPIRUN)
               AC_SUBST(MPI_INCLUDE)]
               , [$1])
    else
        ifelse([$2], , :, [$2])
    fi
else
    ac_cv_lib_mpi=no
    ifelse([$2], , :, [$2])
fi
]
)
# end of PAC_LIB_MPI
#
# PAC_LIB_MPI_UNI
#
# If MPI is not yet defined, use the uniprocessor MPI
AC_DEFUN(PAC_LIB_MPI_UNI,
[
if test "$ac_cv_lib_mpi" = "no"; then
    MPI_LIB="-lmpiuni"
    MPI_INCLUDE="-I$PETSC_DIR/src/sys/src/mpiuni"
    MPIRUN="$PETSC_DIR/src/sys/src/mpiuni/mpirun"

    save_CPPFALGS="$CPPFLAGS"
    CPPFLAGS="$MPI_INCLUDE"
    AC_CHECK_HEADER(mpi.h, ,[
        AC_MSG_WARN([configure expected to find PETSc's mpi.h (for running in single processor mode) in
                     $PETSC_DIR/src/sys/src/mpiuni/mpi.h. Check that you have PETSC_DIR set correctly
                     and that the PETSc source tree has been correctly unpacked (for example, check that
                     the extraction was not terminated due to low disk space).])
        AC_MSG_ERROR([Could not find PETSc's mpi.h file!])]
    )
    CPPFLAGS="$save_CPPFLAGS"
    ac_cv_lib_mpi=yes

    PAC_LIB_MPI_AUX

    AC_DEFINE(PETSC_HAVE_MPI_UNI)
    AC_SUBST(MPI_LIB)
    AC_SUBST(MPIRUN)
    AC_SUBST(MPI_INCLUDE)
fi
])
# end of PAC_LIB_MPI_UNI
#
# PAC_LIB_LAPACK([ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
#
# Check for the existence of LAPACK
AC_DEFUN(PAC_LIB_LAPACK,
[
AC_ARG_WITH(lapack, [  --with-lapack               Choose LAPACK implementation], [LAPACK_LIB=$withval])
if test -n "$LAPACK_LIB"; then
    if test "$LAPACK_LIB" = "yes"; then
        USE_LAPACK="yes"
        LAPACK_LIB="lapack"
    elif test "$LAPACK_LIB" = "no"; then
        USE_LAPACK="no"
        LAPACK_LIB="lapack"
    fi
fi
if test "$USE_LAPACK" = "yes"; then 
    AC_REQUIRE([PAC_LIB_BLAS])
    ac_cv_lib_lapack=no;

    if test -n "$LAPACK_LIB" && test ! "$LAPACK_LIB" = "yes"; then
        if test ! -f "$LAPACK_LIB" ; then
            AC_MSG_ERROR([LAPACK library $LAPACK_LIB does not exist!])
        else
            LAPACK_DIR=`dirname $LAPACK_LIB`
            dnl Of course the Autobook says you can use @<:@ and @:>@, but they do not work
            changequote(<<,>>)dnl
            LAPACK_LIB=`basename $LAPACK_LIB | sed -e 's/^lib\([a-zA-Z\_\-]*\)\.\(a\|so\)/\1/'`
            changequote([,])dnl
        fi
    else
        LAPACK_LIB=lapack
    fi

    # Check in LAPACK_LIB
    PAC_CHECK_FORTRAN_LIB($LAPACK_LIB, dtrtrs, [AC_DEFINE(HAVE_LIBLAPACK)], , [-L$LAPACK_DIR])

    # Check in PETSc distribution
    if eval "test \"`echo '$''{ac_cv_fortran_lib_'$LAPACK_LIB'_dtrtrs}'`\" = no"; then
        LAPACK_DIR="$PETSC_DIR/../blaslapack/lib/lib$BOPT/$PETSC_ARCH"
        PAC_CHECK_FORTRAN_LIB($LAPACK_LIB, dtrtrs, [AC_DEFINE(HAVE_LIBLAPACK)], , [-L$LAPACK_DIR])
    fi

    # End of tests using Fortran symbols from C
    if eval "test \"`echo '$''{ac_cv_fortran_lib_'$LAPACK_LIB'_dtrtrs}'`\" = yes"; then
        ac_cv_lib_lapack=yes;
    fi

    if test "$ac_cv_lib_lapack" = "yes"; then
        ifelse([$1], , [
            if test -n "$LAPACK_LIB"; then
                LAPACK_LIB="-l$LAPACK_LIB"
                LIBS="$LAPACK_LIB $LIBS"
            fi
            if test -n "$LAPACK_DIR"; then
                LDFLAGS="$LDFLAGS -L$LAPACK_DIR"
                DYLIBPATH="$DYLIBPATH:$LAPACK_DIR"
            fi
            AC_SUBST(LAPACK_DIR) 
            AC_SUBST(LAPACK_LIB)], [$1])
    else
        ifelse([$2], , :, [$2])
    fi

else

    ac_cv_lib_lapack=no
    ifelse([$2], , :, [$2])
fi
]
)
# end of PAC_LIB_LAPACK
#
# PAC_LIB_BLAS([ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
#
# Check for the existence of BLAS
AC_DEFUN(PAC_LIB_BLAS,
[
AC_ARG_WITH(blas, [  --with-blas                 Choose BLAS implementation], [BLAS_LIB=$withval])
if test -n "$BLAS_LIB"; then
    if test "$BLAS_LIB" = "yes"; then
        USE_BLAS="yes"
        BLAS_LIB="blas"
    elif test "$BLAS_LIB" = "no"; then
        USE_BLAS="no"
        BLAS_LIB="blas"
    fi
fi
if test "$USE_BLAS" = "yes" ; then 
    AC_REQUIRE([PAC_ENV_PETSC])
    AC_REQUIRE([AC_F77_LIBRARY_LDFLAGS])
    ac_cv_lib_blas=no;

    if test -n "$BLAS_LIB"; then
        if test ! -f "$BLAS_LIB" ; then
            AC_MSG_ERROR([BLAS library $BLAS_LIB does not exist!])
        else
            BLAS_DIR=`dirname $BLAS_LIB`
            dnl Of course the Autobook says you can use @<:@ and @:>@, but they do not work
            changequote(<<,>>)dnl
            BLAS_LIB=`basename $BLAS_LIB | sed -e 's/^lib\([a-zA-Z\_\-]*\)\.\(a\|so\)/\1/'`
            changequote([,])dnl
        fi
    else
        BLAS_DIR="/usr/lib"
        BLAS_LIB="blas"
    fi

    # Check in BLAS_LIB
    PAC_CHECK_FORTRAN_LIB($BLAS_LIB, ddot, [AC_DEFINE(HAVE_LIBBLAS)], , [-L$BLAS_DIR $FLIBS])

    # Check in PETSc distribution
    if eval "test \"`echo '$''{ac_cv_fortran_lib_'$BLAS_LIB'_ddot}'`\" = no"; then
        BLAS_DIR="$PETSC_DIR/../blaslapack/lib/lib$BOPT/$PETSC_ARCH"
        PAC_CHECK_FORTRAN_LIB($BLAS_LIB, ddot, [AC_DEFINE(HAVE_LIBBLAS)], , [-L$BLAS_DIR $FLIBS])
    fi

    # End of tests using Fortran symbols from C
    if eval "test \"`echo '$''{ac_cv_fortran_lib_'$BLAS_LIB'_ddot}'`\" = yes"; then
        ac_cv_lib_blas=yes;
    fi

    # Check for functions embedded in a system library like sunperf (which needs a stupid link line)
    if test "$ac_cv_lib_blas" = "no"; then
        save_LIBS="$LIBS"
        LIBS="$LIBS -xlic_lib=sunperf"
        AC_CHECK_FUNC(ddot, ac_cv_lib_blas_sunperf=yes, ac_cv_lib_blas_sunperf=no)
        if test "$ac_cv_lib_blas_sunperf" = "yes"; then
            ac_cv_lib_blas_ddot=yes
            BLAS_LIB=""
            BLAS_DIR=""
        else
            LIBS="$save_LIBS"
        fi
    fi

    if test "$ac_cv_lib_blas" = "yes"; then
        ifelse([$1], , [
            if test -n "$BLAS_LIB"; then
                BLAS_LIB="-l$BLAS_LIB $FLIBS"
                LIBS="$BLAS_LIB $LIBS"
            fi
            if test -n "$BLAS_DIR"; then
                LDFLAGS="-L$BLAS_DIR $LDFLAGS"
                DYLIBPATH="$DYLIBPATH:$BLAS_DIR"
            fi
            AC_SUBST(BLAS_DIR) 
            AC_SUBST(BLAS_LIB)], [$1])
    else
        ifelse([$2], , :, [$2])
    fi

else

    ac_cv_lib_blas=no
    ifelse([$2], , :, [$2])
fi
]
)
# end of PAC_LIB_BLAS
# PAC_CHECK_SIZEOF(TYPE, [CROSS-SIZE], [HEADER])
#
# Check the sizeof a type giving an optional header
AC_DEFUN(PAC_CHECK_SIZEOF,
[
changequote(<<, >>)dnl
dnl The name to #define.
define(<<AC_TYPE_NAME>>, translit(sizeof_$1, [a-z *], [A-Z_P]))dnl
dnl The cache variable name.
define(<<AC_CV_NAME>>, translit(ac_cv_sizeof_$1, [ *], [_p]))dnl
changequote([, ])dnl
AC_MSG_CHECKING(size of $1)
AC_CACHE_VAL(AC_CV_NAME,
[AC_TRY_RUN([#include <stdio.h>]
ifelse([$3], , , [#include $3])
[main()
{
  FILE *f=fopen("conftestval", "w");
  if (!f) exit(1);
  fprintf(f, "%d\n", sizeof($1));
  exit(0);
}], AC_CV_NAME=`cat conftestval`, AC_CV_NAME=0, ifelse([$2], , , AC_CV_NAME=$2))])dnl
AC_MSG_RESULT($AC_CV_NAME)
AC_DEFINE_UNQUOTED(AC_TYPE_NAME, $AC_CV_NAME)
undefine([AC_TYPE_NAME])dnl
undefine([AC_CV_NAME])dnl
]
)
# end of PAC_CHECK_SIZEOF
